@@ -2,6 +2,7 @@
 
 import math
 import os
+import time
 from collections import deque
 from typing import TYPE_CHECKING
 
@@ -156,6 +157,12 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
         assert self.max_batch_tkv_limit != '-1', (
             "Expecting the env var VLLM_DT_MAX_BATCH_TKV_LIMIT to be set in "
             "platform.py")
+        # cache for self.check_batch_tkv_limit() outer key: tuple(request_ids),
+        # inner key: (request_id, max_batch_tkv_limit), value: (lower, upper)
+        self._cache_check_batch_tkv_limit: dict[tuple, dict[tuple, tuple]] = {}
+        # if batch_is_locked: finish current decode batch to serve a request
+        # that waited for longer than VLLM_SPYRE_MAX_WAITING_TIME_SECONDS
+        self.batch_is_locked = False
 
     def update_from_output(
         self,
@@ -179,6 +186,11 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
         To avoid additional specialization, some requests are held back from the
         base scheduler but are restored after.
         """
+        # unlock the current decode batch if no requests are in running queue
+        if len(self.running) == 0 and self.batch_is_locked:
+            self.batch_is_locked = False
+            logger.debug("Unlocking the current decode batch as no requests "
+                         "are in running queue")
         # First purge the full waiting queue into our holdback queue, preserving
         # priority
         while self.waiting:
@@ -221,11 +233,29 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
         max_prompt_batch_size = 1
         max_context_len = self.scheduler_config.max_model_len
 
+        # if the batch is locked by a request which has been waiting for
+        # longer then VLLM_SPYRE_MAX_WAITING_TIME_SECONDS, we cannot
+        # schedule the current sequence until we have served this request
+        if self.batch_is_locked:
+            return False
+
         # running and waiting queues are both empty -> start a new batch
         # which can always be scheduled
         if len(self.running) + len(self.waiting) == 0:
             return True
 
+        # scheduling heuristic: maximal waiting (blocking) time for prefill
+        waiting_time = (time.monotonic() - request.arrival_time)
+        if waiting_time > envs_spyre.VLLM_SPYRE_MAX_WAITING_TIME_SECONDS:
+            self.batch_is_locked = True
+            logger.debug("Request %s waited longer (%ss) than " \
+            "VLLM_SPYRE_MAX_WAITING_TIME_SECONDS (%ss): locking current " \
+            "decode batch and schedule this request either as part of " \
+            "the current batch or in an exclusive subsequent new batch.",
+            request.request_id, round(waiting_time, 2),
+            envs_spyre.VLLM_SPYRE_MAX_WAITING_TIME_SECONDS
+            )
+
         # check that there is space in the current decode batch
         cond1 = len(self.running) + len(
             self.waiting) < self.max_num_running_reqs
@@ -246,14 +276,28 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
             (self.tkv - request.num_prompt_tokens) / self.block_size)
         num_blocks_required -= num_fully_padded_blocks
         cond5 = num_blocks_required <= self.n_free_blocks
+        # scheduling heuristic: prefill vs decode prioritization
+        # note that prefills are performed on the minimal number of blocks
+        # needed and prefill time is thus proportional to the number of blocks
+        num_blocks_prefill = math.ceil(
+            self.tkv / self.block_size) - num_fully_padded_blocks
+        # if VLLM_SPYRE_N_TOKENS_PREFILL_PRIO is -1 -> no heuristic is enforced
+        cond6 = (num_blocks_prefill * self.block_size
+                 <= envs_spyre.VLLM_SPYRE_N_TOKENS_PREFILL_PRIO) if (
+                     envs_spyre.VLLM_SPYRE_N_TOKENS_PREFILL_PRIO
+                     >= 0) else True
         # check that batch size x tkv is smaller than the max supported number
-        cond6 = self.check_batch_tkv_limit(request=request, tkv=self.tkv)
+        cond7 = lambda: self.check_batch_tkv_limit(request=request,
+                                                   tkv=self.tkv,
+                                                   running=self.running,
+                                                   max_batch_tkv_limit=self.
+                                                   max_batch_tkv_limit)
 
-        if cond1 and cond2 and cond3 and cond4 and cond5 and cond6:
+        if cond1 and cond2 and cond3 and cond4 and cond5 and cond6 and cond7():
             return True
 
         # the following conditions must always be true, if not we can exit here
-        if not (cond1 and cond2 and cond4 and cond5 and cond6
+        if not (cond1 and cond2 and cond4 and cond5 and cond6 and cond7()
                 ) or not envs_spyre.VLLM_SPYRE_ENABLE_PREFILL_OPTIMIZATION:
             return False
 
@@ -279,14 +323,30 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
             (tkv_updated + request.max_tokens - 1) / self.block_size)
         cond5_updated = num_blocks_required_updated <= self.n_free_blocks
 
+        # check prefill vs decode prioritization with updated tkv
+        # Note: num_fully_padded_blocks is always 0 in this code branch by
+        # construction: if the prompt is bigger than self.tkv, we shift
+        # self.tkv by tkv_offset to just accommodate the new prompt. The
+        # alignment with self.tkv this will require max block_size - 1 pads.
+        num_blocks_prefill_updated = math.ceil(tkv_updated / self.block_size)
+        cond6_updated = (num_blocks_prefill_updated * self.block_size
+                         <= envs_spyre.VLLM_SPYRE_N_TOKENS_PREFILL_PRIO) if (
+                             envs_spyre.VLLM_SPYRE_N_TOKENS_PREFILL_PRIO
+                             >= 0) else True
+
         # check that batch size x tkv is smaller than the max supported number
-        # with updated tkv (cond6)
-        cond6_updated = self.check_batch_tkv_limit(request=request,
-                                                   tkv=tkv_updated)
+        # with updated tkv (cond6) -> only call if the other cond are met
+        cond7_updated = lambda: self.check_batch_tkv_limit(
+            request=request,
+            tkv=tkv_updated,
+            running=self.running,
+            max_batch_tkv_limit=self.max_batch_tkv_limit)
 
-        return cond4_updated and cond5_updated and cond6_updated
+        return (cond4_updated and cond5_updated and cond6_updated
+                and cond7_updated())
 
-    def check_batch_tkv_limit(self, request, tkv) -> bool:
+    def check_batch_tkv_limit(self, request, tkv, running,
+                              max_batch_tkv_limit) -> bool:
         """
         Check whether adding a new sequence to the decode batch would violate
         Spyre's maximum batch volume constraint.
@@ -308,10 +368,20 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
 
         Note: drawing explaining the algorithm in more detail uploaded here: 
         https://github.com/vllm-project/vllm-spyre/pull/363#issuecomment-3173605517
-        
-        WIP: The result of this check could be cached and reused if both the 
-        decode batch and the new input request are unchanged between calls.
         """
+        # checking if cached result can be used
+        outer_key = tuple(r.request_id
+                          for r in running)  # decode batch changes
+        inner_key = (request.request_id, max_batch_tkv_limit
+                     )  # new request changes
+        cache = self._cache_check_batch_tkv_limit
+        if (outer_key in cache) and (inner_key in cache[outer_key]):
+            (lower, upper) = cache[outer_key][inner_key]
+            if tkv <= lower or tkv >= upper:
+                logger.debug(
+                    "Cache hit function check_batch_tkv_limit: returning %s",
+                    str(tkv <= lower))
+                return tkv <= lower
 
         # Compute the effective token length of the new request
         new_req_tkv = tkv + request.max_tokens - 1
@@ -320,13 +390,13 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
         decode_req_tkvs = [
             tkv + req.max_tokens - 1 -
             (req.num_computed_tokens - req.num_prompt_tokens)
-            for req in self.running
+            for req in running
         ]
         # Sort decode requests token lengths in ascending order
         decode_req_tkvs.sort()
 
         # Initialize values
-        batch_size = len(self.running) + 1
+        batch_size = len(running) + 1
         max_batch_tkv = 0
 
         # Try adding the new request to the batch and check the max volume
@@ -341,4 +411,31 @@ class ContinuousBatchingSpyreScheduler(SpyreScheduler):
                 # decrease batch_size by 1 as the current request finished
                 batch_size -= 1
 
-        return max_batch_tkv <= int(self.max_batch_tkv_limit)
+        return_value = max_batch_tkv <= int(max_batch_tkv_limit)
+
+        if outer_key in cache:
+            # decode batch has not changed
+            if inner_key not in cache[outer_key]:
+                # adding new request to present decode batch
+                cache[outer_key][inner_key] = (-math.inf, math.inf)
+        else:
+            # decode batch has changed, empty the cache in place
+            cache.clear()
+            cache[outer_key] = {inner_key: (-math.inf, math.inf)}
+            logger.debug(
+                "Cleared cache of function check_batch_tkv_limit as the " \
+                "decode batch has changed."
+            )
+
+        # update lower bound (of acceptance) and upper bound (of rejection)
+        (lower, upper) = cache[outer_key][inner_key]
+        if return_value:
+            lower = max(lower, tkv)
+        else:
+            upper = min(upper, tkv)
+        assert lower < upper
+        cache[outer_key][inner_key] = (lower, upper)
+        logger.debug("Saved cache of function check_batch_tkv_limit: %s",
+                     self._cache_check_batch_tkv_limit[outer_key][inner_key])
+
+        return return_value
