@@ -7,6 +7,7 @@ import platform
 import signal
 import sys
 import time
+from datetime import timedelta
 from pathlib import Path
 from typing import Optional, Union, cast
 
@@ -322,11 +323,24 @@ class SpyreWorker(WorkerBaseV1):
 
         if not self._env_initialized:
 
+            backend = "gloo"
+            init_method = "env://"
+
+            # TODO: temporary fix until we have
+            # timeout in vllm's init_distributed_environment
+            torch.distributed.init_process_group(
+                backend=backend,
+                init_method=init_method,
+                world_size=self.parallel_config.world_size,
+                rank=self.rank,
+                timeout=timedelta(
+                    minutes=envs_spyre.VLLM_SPYRE_GLOO_TIMEOUT_MINUTES))
+
             init_distributed_environment(
                 world_size=self.parallel_config.world_size,
                 rank=self.rank,
-                distributed_init_method="env://",
-                backend="gloo",
+                distributed_init_method=init_method,
+                backend=backend,
             )
 
             if self.parallel_config.world_size > 1:
@@ -397,6 +411,7 @@ class SpyreWorker(WorkerBaseV1):
     def _warmup_spyre_dynamic_size(self, special_token_ids):
         # this setting is required to mark a dimension of size 1 as dynamic
         # for pytorch >= 2.7.1 (needed to support batch size 1 for decodes)
+
         from torch.fx.experimental import _config as config
         config.backed_size_oblivious = True
 
@@ -421,19 +436,30 @@ class SpyreWorker(WorkerBaseV1):
 
         # Sample from the valid token ids
         warmup_tokens_tensor = valid_token_ids_tensor[torch.randint(
-            0, len(valid_token_ids_tensor), (2, prompt_len))]
+            0, len(valid_token_ids_tensor), (3, prompt_len))]
 
-        warmup_req, deploy_req = (new_request_data_builder(
-            req_id="warmup-%d" % (i),
-            prompt_token_ids=warmup_tokens_tensor[i].tolist(),
-            sampling_params=SamplingParams(max_tokens=num_decode_tokens),
-            pooling_params=None,
-        ) for i in range(2))
+        # TODO: we need 2 requests for warmup on FP8+CB
+        is_fp8_plus_cb = 'FP8' in self.model_config.model and \
+            envs_spyre.VLLM_SPYRE_USE_CB
+        req_count = 3 if is_fp8_plus_cb else 2
+        requests = [
+            new_request_data_builder(
+                req_id="warmup-%d" % (i),
+                prompt_token_ids=warmup_tokens_tensor[i].tolist(),
+                sampling_params=SamplingParams(max_tokens=num_decode_tokens),
+                pooling_params=None,
+            ) for i in range(req_count)
+        ]
 
+        warmup_requests = requests[:-1]  # first one or two
+        deploy_req = requests[-1]  # Last one
         model_runner.pre_warmup()
+
         with _maybe_warmup_context(envs_spyre.VLLM_SPYRE_MAX_LOAD_PROCESSES,
                                    self.parallel_config.world_size, self.rank):
-            self._dynamic_warmup(request=warmup_req,
+            # TODO(wallas): I am not sure if really need warmup with at
+            # least batch size 2 for quantized model
+            self._dynamic_warmup(requests=warmup_requests,
                                  prompt_len=prompt_len,
                                  valid_token_ids_tensor=valid_token_ids_tensor)
 
@@ -606,47 +632,61 @@ class SpyreWorker(WorkerBaseV1):
 
     def _dynamic_warmup(
         self,
-        request: NewRequestData,
+        requests: list[NewRequestData],
         prompt_len: int,
         valid_token_ids_tensor: torch.Tensor,
     ) -> None:
 
+        # TODO: because of FP8 we are doing warmup with bs=2 again.
+        # Once we figure it out this limitation we should revert this to
+        # bs=1 again.
         assert (
             _inside_warmup_mode
         ), "it looks like you are outside the warmup context for warmup"
 
-        scheduler_output = SchedulerOutput(
-            scheduled_new_reqs=[request],
-            scheduled_cached_reqs=CachedRequestData.make_empty(),
-            num_scheduled_tokens={request.req_id: prompt_len},
-            total_num_scheduled_tokens=prompt_len,
-            scheduled_spec_decode_tokens={},
-            scheduled_encoder_inputs={},
-            num_common_prefix_blocks=0,
-            finished_req_ids=set(),
-            free_encoder_input_ids=[],
-            structured_output_request_ids={},
-            grammar_bitmask=None,
-        )
-        logger.info("[WARMUP] Prefill...")
+        req_count = len(requests)
+        for idx, req in enumerate(requests):
+            scheduler_output = SchedulerOutput(
+                scheduled_new_reqs=[req],
+                scheduled_cached_reqs=CachedRequestData.make_empty(),
+                num_scheduled_tokens={req.req_id: prompt_len},
+                total_num_scheduled_tokens=prompt_len,
+                scheduled_spec_decode_tokens={},
+                scheduled_encoder_inputs={},
+                num_common_prefix_blocks=0,
+                finished_req_ids=set(),
+                free_encoder_input_ids=[],
+                structured_output_request_ids={},
+                grammar_bitmask=None,
+            )
 
-        self.execute_model(scheduler_output)
+            logger.info("[WARMUP] Prefill [%s/%s]...", idx + 1, req_count)
+
+            self.execute_model(scheduler_output)
+
+
+        random_token_id = \
+            lambda: torch.randint(0, len(valid_token_ids_tensor), (1, )).item()
+
+        # Reduce to accumulate all blocks
+        block_ids : list[int] = \
+            functools.reduce(lambda blocks, req: blocks + req.block_ids,
+                             requests, [])
 
         cached_request_data = CachedRequestData(
-            req_ids=[request.req_id],
+            req_ids=[req.req_id for req in requests],
             resumed_from_preemption=False,
-            new_token_ids=[[
-                valid_token_ids_tensor[torch.randint(
-                    0, len(valid_token_ids_tensor), (1, )).item()]
-            ]],
-            new_block_ids=[request.block_ids],
-            num_computed_tokens=[prompt_len],
+            new_token_ids=[[valid_token_ids_tensor[random_token_id()]]
+                           for _ in requests],
+            new_block_ids=block_ids,
+            num_computed_tokens=[prompt_len for _ in requests],
         )
 
         scheduler_output = SchedulerOutput(
             scheduled_new_reqs=[],
             scheduled_cached_reqs=cached_request_data,
-            num_scheduled_tokens={request.req_id: 1},
+            num_scheduled_tokens={req.req_id: 1
+                                  for req in requests},
             total_num_scheduled_tokens=1,
             scheduled_spec_decode_tokens={},
             scheduled_encoder_inputs={},
@@ -658,7 +698,7 @@ class SpyreWorker(WorkerBaseV1):
         )
         logger.info("[WARMUP] Decode...")
         self.execute_model(scheduler_output)
-        self._cleanup_model_runner(request=[request])
+        self._cleanup_model_runner(request=requests)
 
     def _warmup_model_forward_pass(
         self,
